searchNodes=[{"doc":"Normalizer Normalizes string-keyed maps to atom-keyed maps while converting values according to a given schema. Particularly useful when working with param maps. Usage schema = %{ user_id : { :number , required : true } , name : :string , admin : { :boolean , default : false } , languages : [ :string ] } params = %{ &quot;user_id&quot; =&gt; &quot;42&quot; , &quot;name&quot; =&gt; &quot;Neo&quot; , &quot;languages&quot; =&gt; [ &quot;en&quot; ] , &quot;age&quot; =&gt; &quot;55&quot; } &gt; Normalizer . normalize ( params , schema ) %{ user_id : 42 , name : &quot;Neo&quot; , admin : false , languages : [ &quot;en&quot; ] } Properties Converts types whenever possible and reasonable; Ensures required values are given; Filters keys and values not in the schema; Supports basic types, lists, maps, and nested lists/maps. See Normalizer.normalize/2 for more details.","ref":"Normalizer.html","title":"Normalizer","type":"module"},{"doc":"Normalizes a string-map using the given schema. The schema is expected to be a map where each key is an atom representing an expected string key in params , pointing to the type to which the respective value in the params should be normalized. The return is a normalized map, in case of success, or a map with each erroring key and a description, in case of failure. Types Types can be one of: Primitives : :string , :number , :boolean . Parseable values : :datetime , :date . Maps : a nested schema for a nested map. Lists : one-element lists that contain any of the other types. Tuples : two-element tuples where the first element is one of the other types, and the second element a keyword list of options. Primitives Strings are somewhat of a catch all that ensures anything but lists and maps are converted to strings: iex&gt; Normalizer . normalize ( %{ &quot;key&quot; =&gt; 42 } , %{ key : :string } ) { :ok , %{ key : &quot;42&quot; } } Numbers are kept as is, if possible, or parsed: iex&gt; Normalizer . normalize ( %{ &quot;key&quot; =&gt; 42 } , %{ key : :number } ) { :ok , %{ key : 42 } } iex&gt; Normalizer . normalize ( %{ &quot;key&quot; =&gt; &quot;42.5&quot; } , %{ key : :number } ) { :ok , %{ key : 42.5 } } Booleans accept native values, &quot;true&quot; and &quot;false&quot; strings, and &quot;1&quot; and &quot;0&quot; strings: iex&gt; Normalizer . normalize ( %{ &quot;key&quot; =&gt; true } , %{ key : :boolean } ) { :ok , %{ key : true } } iex&gt; Normalizer . normalize ( %{ &quot;key&quot; =&gt; &quot;false&quot; } , %{ key : :boolean } ) { :ok , %{ key : false } } Parseable Values Only :datetime and :date are supported right now. iex&gt; Normalizer . normalize ( %{ &quot;key&quot; =&gt; &quot;2020-02-11T00:00:00+0100&quot; } , %{ key : :datetime } ) { :ok , %{ key : ~U[2020-02-10T23:00:00Z] } } iex&gt; Normalizer . normalize ( %{ &quot;key&quot; =&gt; &quot;2020-02-11&quot; } , %{ key : :date } ) { :ok , %{ key : ~D[2020-02-11] } } The offset can be extracted as well by passing the with_offset option: iex&gt; Normalizer . normalize ( ...&gt; %{ &quot;key&quot; =&gt; &quot;2020-02-11T00:00:00+0100&quot; } , ...&gt; %{ key : { :datetime , with_offset : true } } ...&gt; ) { :ok , %{ key : { ~U[2020-02-10T23:00:00Z] , 3600 } } } Maps Nested schemas are supported: iex&gt; Normalizer . normalize ( ...&gt; %{ &quot;key&quot; =&gt; %{ &quot;age&quot; =&gt; &quot;42&quot; } } , ...&gt; %{ key : %{ age : :number } } ...&gt; ) { :ok , %{ key : %{ age : 42 } } } Lists Lists are represented in the schema by a single-element list: iex&gt; Normalizer . normalize ( ...&gt; %{ &quot;key&quot; =&gt; [ &quot;42&quot; , 52 ] } , ...&gt; %{ key : [ :number ] } ...&gt; ) { :ok , %{ key : [ 42 , 52 ] } } We can normalize lists of any one of the other types. Options Per-value options can be specified by passing a two-element tuple in the type specification. The three available options are :required , :default , and :with_offset . :required fails the validation process if the key is missing or nil: iex&gt; Normalizer . normalize ( %{ &quot;key&quot; =&gt; nil } , %{ key : :number } ) { :ok , %{ key : nil } } iex&gt; Normalizer . normalize ( %{ &quot;key&quot; =&gt; nil } , %{ key : { :number , required : true } } ) { :error , %{ key : &quot;required number, got nil&quot; } } iex&gt; Normalizer . normalize ( %{ } , %{ key : { :number , required : true } } ) { :error , %{ key : &quot;required number&quot; } } :default ensures a value in a given key, if nil or missing: iex&gt; Normalizer . normalize ( %{ } , %{ key : { :number , default : 42 } } ) { :ok , %{ key : 42 } } iex&gt; Normalizer . normalize ( %{ &quot;key&quot; =&gt; 24 } , %{ key : { :number , default : 42 } } ) { :ok , %{ key : 24 } } :with_offset is explained in the :datetime type above.","ref":"Normalizer.html#normalize/2","title":"Normalizer.normalize/2","type":"function"},{"doc":"","ref":"Normalizer.html#t:schema/0","title":"Normalizer.schema/0","type":"type"},{"doc":"","ref":"Normalizer.html#t:value_options/0","title":"Normalizer.value_options/0","type":"type"},{"doc":"","ref":"Normalizer.html#t:value_schema/0","title":"Normalizer.value_schema/0","type":"type"},{"doc":"","ref":"Normalizer.html#t:value_type/0","title":"Normalizer.value_type/0","type":"type"},{"doc":"Apache License Version 2.0, January 2004 http://www.apache.org/licenses/ TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION 1. Definitions. &quot;License&quot; shall mean the terms and conditions for use, reproduction, and distribution as defined by Sections 1 through 9 of this document. &quot;Licensor&quot; shall mean the copyright owner or entity authorized by the copyright owner that is granting the License. &quot;Legal Entity&quot; shall mean the union of the acting entity and all other entities that control, are controlled by, or are under common control with that entity. For the purposes of this definition, &quot;control&quot; means (i) the power, direct or indirect, to cause the direction or management of such entity, whether by contract or otherwise, or (ii) ownership of fifty percent (50%) or more of the outstanding shares, or (iii) beneficial ownership of such entity. &quot;You&quot; (or &quot;Your&quot;) shall mean an individual or Legal Entity exercising permissions granted by this License. &quot;Source&quot; form shall mean the preferred form for making modifications, including but not limited to software source code, documentation source, and configuration files. &quot;Object&quot; form shall mean any form resulting from mechanical transformation or translation of a Source form, including but not limited to compiled object code, generated documentation, and conversions to other media types. &quot;Work&quot; shall mean the work of authorship, whether in Source or Object form, made available under the License, as indicated by a copyright notice that is included in or attached to the work (an example is provided in the Appendix below). &quot;Derivative Works&quot; shall mean any work, whether in Source or Object form, that is based on (or derived from) the Work and for which the editorial revisions, annotations, elaborations, or other modifications represent, as a whole, an original work of authorship. For the purposes of this License, Derivative Works shall not include works that remain separable from, or merely link (or bind by name) to the interfaces of, the Work and Derivative Works thereof. &quot;Contribution&quot; shall mean any work of authorship, including the original version of the Work and any modifications or additions to that Work or Derivative Works thereof, that is intentionally submitted to Licensor for inclusion in the Work by the copyright owner or by an individual or Legal Entity authorized to submit on behalf of the copyright owner. For the purposes of this definition, &quot;submitted&quot; means any form of electronic, verbal, or written communication sent to the Licensor or its representatives, including but not limited to communication on electronic mailing lists, source code control systems, and issue tracking systems that are managed by, or on behalf of, the Licensor for the purpose of discussing and improving the Work, but excluding communication that is conspicuously marked or otherwise designated in writing by the copyright owner as &quot;Not a Contribution.&quot; &quot;Contributor&quot; shall mean Licensor and any individual or Legal Entity on behalf of whom a Contribution has been received by Licensor and subsequently incorporated within the Work. 2. Grant of Copyright License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable copyright license to reproduce, prepare Derivative Works of, publicly display, publicly perform, sublicense, and distribute the Work and such Derivative Works in Source or Object form. 3. Grant of Patent License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable (except as stated in this section) patent license to make, have made, use, offer to sell, sell, import, and otherwise transfer the Work, where such license applies only to those patent claims licensable by such Contributor that are necessarily infringed by their Contribution(s) alone or by combination of their Contribution(s) with the Work to which such Contribution(s) was submitted. If You institute patent litigation against any entity (including a cross-claim or counterclaim in a lawsuit) alleging that the Work or a Contribution incorporated within the Work constitutes direct or contributory patent infringement, then any patent licenses granted to You under this License for that Work shall terminate as of the date such litigation is filed. 4. Redistribution. You may reproduce and distribute copies of the Work or Derivative Works thereof in any medium, with or without modifications, and in Source or Object form, provided that You meet the following conditions: (a) You must give any other recipients of the Work or Derivative Works a copy of this License; and (b) You must cause any modified files to carry prominent notices stating that You changed the files; and (c) You must retain, in the Source form of any Derivative Works that You distribute, all copyright, patent, trademark, and attribution notices from the Source form of the Work, excluding those notices that do not pertain to any part of the Derivative Works; and (d) If the Work includes a &quot;NOTICE&quot; text file as part of its distribution, then any Derivative Works that You distribute must include a readable copy of the attribution notices contained within such NOTICE file, excluding those notices that do not pertain to any part of the Derivative Works, in at least one of the following places: within a NOTICE text file distributed as part of the Derivative Works; within the Source form or documentation, if provided along with the Derivative Works; or, within a display generated by the Derivative Works, if and wherever such third-party notices normally appear. The contents of the NOTICE file are for informational purposes only and do not modify the License. You may add Your own attribution notices within Derivative Works that You distribute, alongside or as an addendum to the NOTICE text from the Work, provided that such additional attribution notices cannot be construed as modifying the License. You may add Your own copyright statement to Your modifications and may provide additional or different license terms and conditions for use, reproduction, or distribution of Your modifications, or for any such Derivative Works as a whole, provided Your use, reproduction, and distribution of the Work otherwise complies with the conditions stated in this License. 5. Submission of Contributions. Unless You explicitly state otherwise, any Contribution intentionally submitted for inclusion in the Work by You to the Licensor shall be under the terms and conditions of this License, without any additional terms or conditions. Notwithstanding the above, nothing herein shall supersede or modify the terms of any separate license agreement you may have executed with Licensor regarding such Contributions. 6. Trademarks. This License does not grant permission to use the trade names, trademarks, service marks, or product names of the Licensor, except as required for reasonable and customary use in describing the origin of the Work and reproducing the content of the NOTICE file. 7. Disclaimer of Warranty. Unless required by applicable law or agreed to in writing, Licensor provides the Work (and each Contributor provides its Contributions) on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied, including, without limitation, any warranties or conditions of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A PARTICULAR PURPOSE. You are solely responsible for determining the appropriateness of using or redistributing the Work and assume any risks associated with Your exercise of permissions under this License. 8. Limitation of Liability. In no event and under no legal theory, whether in tort (including negligence), contract, or otherwise, unless required by applicable law (such as deliberate and grossly negligent acts) or agreed to in writing, shall any Contributor be liable to You for damages, including any direct, indirect, special, incidental, or consequential damages of any character arising as a result of this License or out of the use or inability to use the Work (including but not limited to damages for loss of goodwill, work stoppage, computer failure or malfunction, or any and all other commercial damages or losses), even if such Contributor has been advised of the possibility of such damages. 9. Accepting Warranty or Additional Liability. While redistributing the Work or Derivative Works thereof, You may choose to offer, and charge a fee for, acceptance of support, warranty, indemnity, or other liability obligations and/or rights consistent with this License. However, in accepting such obligations, You may act only on Your own behalf and on Your sole responsibility, not on behalf of any other Contributor, and only if You agree to indemnify, defend, and hold each Contributor harmless for any liability incurred by, or claims asserted against, such Contributor by reason of your accepting any such warranty or additional liability. END OF TERMS AND CONDITIONS","ref":"license.html","title":"License","type":"extras"},{"doc":"Normalizes string-keyed maps to atom-keyed maps while converting values according to a given schema. Particularly useful when working with param maps.","ref":"readme.html","title":"Readme","type":"extras"},{"doc":"iex&gt; schema = %{ user_id : { :number , required : true } , name : :string , admin : { :boolean , default : false } , languages : [ :string ] } iex&gt; params = %{ &quot;user_id&quot; =&gt; &quot;42&quot; , &quot;name&quot; =&gt; &quot;Neo&quot; , &quot;languages&quot; =&gt; [ &quot;en&quot; ] , &quot;age&quot; =&gt; &quot;55&quot; } iex&gt; Normalizer . normalize ( params , schema ) { :ok , %{ user_id : 42 , name : &quot;Neo&quot; , admin : false , languages : [ &quot;en&quot; ] } }","ref":"readme.html#usage","title":"Readme - Usage","type":"extras"},{"doc":"Converts types whenever possible and reasonable; Ensures required values are given; Filters keys and values not in the schema; Supports basic types, lists, maps, and nested lists/maps. See docs for Normalizer.normalize/2 for more details.","ref":"readme.html#properties","title":"Readme - Properties","type":"extras"},{"doc":"Add normalizer to your list of dependencies in mix.exs : def deps do [ { :normalizer , &quot;~&gt; 0.3.0&quot; } ] end","ref":"readme.html#installation","title":"Readme - Installation","type":"extras"},{"doc":"PRs welcome, unit tests required.","ref":"readme.html#contributing","title":"Readme - Contributing","type":"extras"},{"doc":"Copyright (c) 2022 João Ferreira Licensed under the Apache License, Version 2.0 (the &quot;License&quot;); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.","ref":"readme.html#copyright-and-license","title":"Readme - Copyright and License","type":"extras"}]